
# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: //:liga-sklearn

from setuptools import setup

setup(**{
    'install_requires': (
        'liga==0.2.0.dev11',
        'scikit-learn',
    ),
    'name': 'liga-sklearn',
    'namespace_packages': (
    ),
    'package_data': {
    },
    'packages': (
        'liga.sklearn',
        'liga.sklearn.models',
    ),
    'python_requires': '<3.10,>=3.8',
    'version': '0.2.0.dev11',
})
